@@ -1,18 +1,13 @@
 from datetime import datetime, timezone
 from functools import cached_property, wraps
-from io import BytesIO
-import os
 from pathlib import Path
-from shutil import rmtree
-import tempfile
 from time import sleep
 from typing import Any, Callable, Dict, Iterator, List, Tuple
-from zipfile import ZipFile
 
 from github import Github
-from github.GithubException import RateLimitExceededException
 from github.GitRelease import GitRelease
 from github.GitReleaseAsset import GitReleaseAsset
+from github.GithubException import GithubException, RateLimitExceededException
 from github.Repository import Repository
 from github.Workflow import Workflow
 from github.WorkflowRun import WorkflowRun
@@ -82,7 +77,22 @@ class GitHubActions(CISystem):
     @cached_property  # type: ignore[misc]
     @retry_ratelimit
     def ghrepo(self) -> Repository:
-        return self.client.get_repo(self.repo)
+        i = 0
+        while True:
+            try:
+                return self.client.get_repo(self.repo)
+            except GithubException as e:
+                if e.status == 502 and i < 10:
+                    log.warning(
+                        "Request to fetch %s GitHub repository details returned"
+                        " %d; waiting & retrying",
+                        self.repo,
+                        e.status,
+                    )
+                    i += 1
+                    sleep(i * i)
+                else:
+                    raise
 
     @retry_ratelimit
     def get_workflows(self) -> List[Workflow]:
@@ -315,7 +325,7 @@ class GHABuildLog(GHAAsset, BuildLog):
             path,
         )
         try:
-            r = self.client.get(self.logs_url)
+            self.client.download_zipfile(self.logs_url, path)
         except requests.HTTPError as e:
             if e.response.status_code in (404, 410):
                 # 404 can happen when a workflow failed to run due to, say, a
@@ -326,12 +336,6 @@ class GHABuildLog(GHAAsset, BuildLog):
                 return []
             else:
                 raise
-        try:
-            with BytesIO(r.content) as blob, ZipFile(blob) as zf:
-                zf.extractall(path)
-        except BaseException:
-            rmtree(path)
-            raise
         return list(path.rglob("*.txt"))
 
 
@@ -387,18 +391,7 @@ class GHAArtifact(GHAAsset, Artifact):
             self.number,
             path,
         )
-        fd, fpath = tempfile.mkstemp()
-        os.close(fd)
-        zippath = Path(fpath)
-        self.client.download(self.download_url, zippath)
-        try:
-            with ZipFile(zippath) as zf:
-                zf.extractall(target_dir)
-        except BaseException:
-            rmtree(target_dir)
-            raise
-        finally:
-            zippath.unlink(missing_ok=True)
+        self.client.download_zipfile(self.download_url, target_dir)
         return list(iterfiles(target_dir))
 
 
